// Copyright 2022 Christian Ulrich
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// usage: ./part2 < input

use std::io;

fn accumulated_calories() -> Result<Vec<usize>, &'static str> {
    let mut result = vec![];
    let mut acc = None;
    for line in io::stdin().lines() {
        let line = line.unwrap();
        if line.len() == 0 {
            if let Some(value) = acc {
                result.push(value);
                acc = None;
            }
        } else {
            match line.parse::<usize>() {
                Ok(value) => acc = Some(acc.unwrap_or(0) + value),
                Err(_) => return Err("input file contains invalid line"),
            }
        }
    }
    if let Some(value) = acc {
        result.push(value)
    }
    Ok(result)
}

fn main() -> Result<(), &'static str> {
    let mut calories = accumulated_calories()?;
    calories.sort();
    let top_three = calories.iter().rev().take(3);
    println!("{}", top_three.sum::<usize>());
    Ok(())
}
