// Copyright 2022 Christian Ulrich
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

use std::io::{self, Read};

pub struct DatastreamWindows<const N: usize> {
    window: [u8; N],
}

impl<const N: usize> DatastreamWindows<N> {
    pub fn from_stdin() -> Result<Self, &'static str> {
        let mut window = [0; N];
        io::stdin()
            .read_exact(&mut window[1..N])
            .map_err(|_| "not enough data available")?;
        Ok(Self { window: window })
    }
}

impl<const N: usize> Iterator for DatastreamWindows<N> {
    type Item = [u8; N];

    fn next(&mut self) -> Option<Self::Item> {
        for byte in io::stdin().bytes() {
            let byte = byte.unwrap();
            self.window.rotate_left(1);
            self.window[N - 1] = byte;
            return Some(self.window);
        }
        None
    }
}
